package com.zswl.cloud.springBtach.server.core.api.video;

import cn.hutool.http.HttpStatus;
import com.alibaba.fastjson.JSON;
import com.github.microservice.auth.client.content.ResultContent;
import com.github.microservice.auth.client.content.ResultState;
import com.github.microservice.core.util.net.apache.HttpClientUtil;
import com.github.microservice.core.util.net.apache.HttpModel;
import com.github.microservice.core.util.net.apache.MethodType;
import com.github.microservice.core.util.net.apache.ResponseModel;
import com.zswl.cloud.shop.client.dto.yqd.OrderDto;
import com.zswl.cloud.springBtach.server.core.dao.ApiLogDao;
import com.zswl.cloud.springBtach.server.core.domain.ApiLog;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;

@Service
@Slf4j
public class YqdServiceImpl {


    @Value("${yqd.key}")
    private String key;

    @Value("${yqd.account}")
    private String account;


    @Value("${yqd.createUrl}")
    private String createUrl;

    @Value("${yqd.callbackUrl}")
    private String callbackUrl;

    @Autowired
    ApiLogDao apiLogDao;

    @SneakyThrows
    public ResultContent create(OrderDto dto) {
        long timestamp = System.currentTimeMillis();
        Map<String, Object> body = new HashMap<>();
        body.put("commodityId", dto.getCommodityId());
        body.put("buyCount", dto.getBuyCount());
        body.put("externalSellPrice", dto.getExternalSellPrice());
        body.put("template", "[" + dto.getPhone() + "]");
        String url = String.format(createUrl, timestamp, account, getSign(timestamp, JSON.toJSONString(body)));
        ResponseModel request = HttpClientUtil.request(HttpModel.builder().method(MethodType.Post).url(url).body(JSON.toJSONString(body)).build());

        ApiLog apiLog = new ApiLog();
        apiLog.setService("create");
        apiLog.setSystem("video");
        apiLog.setRequest(JSON.toJSONString(body));
        apiLog.setResponse(JSON.toJSONString(request.getBody()));
        apiLogDao.save(apiLog);

        try {
            LinkedHashMap map = (LinkedHashMap) request.getBody();
            if (HttpStatus.HTTP_OK == (int) map.get("code")) {
                log.info("视频会员响应：{}", JSON.toJSONString(request));
                return ResultContent.build(ResultState.Success, request.getBody());
            } else {
                return ResultContent.build(ResultState.Fail, map.get("msg"));
            }
        } catch (Exception e) {
            return ResultContent.build(ResultState.Fail, JSON.parseObject((String) request.getBody()).getString("msg"));
        }
    }







    @SneakyThrows
    public ResultContent create2(YqdCreateRequestModel createRequestModel) {
        createRequestModel.setCallbackUrl(callbackUrl);
        long timestamp = System.currentTimeMillis();
        String url = String.format(createUrl, timestamp, account, getSign(timestamp, JSON.toJSONString(createRequestModel)));
        ResponseModel request = HttpClientUtil.request(HttpModel.builder().method(MethodType.Post).url(url).body(JSON.toJSONString(createRequestModel)).build());

        ApiLog apiLog = new ApiLog();
        apiLog.setService("create");
        apiLog.setSystem("video");
        apiLog.setRequest(JSON.toJSONString(createRequestModel));
        apiLog.setResponse(JSON.toJSONString(request.getBody()));
        apiLogDao.save(apiLog);

        try {
            LinkedHashMap map = (LinkedHashMap) request.getBody();
            if (HttpStatus.HTTP_OK == (int) map.get("code")) {
                log.info("视频会员响应：{}", JSON.toJSONString(request));
                return ResultContent.build(ResultState.Success, request.getBody());
            } else {
                return ResultContent.build(ResultState.Fail, map.get("msg"));
            }
        } catch (Exception e) {
            return ResultContent.build(ResultState.Fail, JSON.parseObject((String) request.getBody()).getString("msg"));
        }
    }


    private String getSign(long timestamp, String data) {
        if (StringUtils.isEmpty(data)) {
            return DigestUtils.md5Hex(timestamp + key);
        }
        return DigestUtils.md5Hex(timestamp + data + key);
    }

}
